import os

import utils as utils


default_yml_item = """
- src: '%src'
  name: '%name'
  scm: '%scm'
  version: '%version'
"""


class Reqs(object):
    """
    Create a requirements file meant to hold a list of all of your roles.
    """
    def __init__(self, args, options, config):
        self.roles = utils.roles_dict(args[0])
        self.out_file = options.out_file

        utils.exit_if_no_roles(len(self.roles.keys()), args[0])

        if options.format:
            self.format = options.format
        else:
            self.format = config["options"]["reqs_format"]

        self.config = config
        self.src = config["scm"]["host"]
        self.scm = config["scm"]["type"]
        self.user = config["scm"]["user"]
        self.repo_prefix = config["scm"]["repo_prefix"]
        self.interactive_version = options.interactive_version

        if self.format == "yml" or self.format == "yaml":
            self.export_yml_reqs()
        else:
            self.export_txt_reqs()

    def export_yml_reqs(self):
        """
        Export the yaml version to stdout or requirements file.
        """
        role_lines = "---\n"

        for galaxy_role, role in sorted(self.roles.items()):
            yml_item = default_yml_item

            if self.src:
                galaxy_name = utils.normalize_role(role, self.config)
                yml_item = yml_item.replace("%name",
                                            "{0}.{1}".format(self.user,
                                                             galaxy_name))

                if self.repo_prefix:
                    role = self.repo_prefix + role

                src = os.path.join(self.src, self.user, role)
            else:
                src = galaxy_role
                yml_item = yml_item.replace("  name: '%name'\n", "")
                yml_item = yml_item.replace("  scm: '%scm'\n", "")

            yml_item = yml_item.replace("%src", src)

            if self.scm:
                yml_item = yml_item.replace("%scm", self.scm)
            else:
                yml_item = yml_item.replace("scm: '%scm'\n", "")

            version = self.get_user_version(role)
            if version:
                yml_item = yml_item.replace("%version", version)
            else:
                yml_item = yml_item.replace("  version: '%version'\n", "")

            role_lines += "{0}".format(yml_item)

        if self.out_file:
            utils.string_to_file(self.out_file, role_lines)
        else:
            print role_lines

    def export_txt_reqs(self):
        """
        Export the text version to stdout or requirements file.
        """
        role_lines = ""

        for role in sorted(self.roles.keys()):
            version = self.get_user_version(role)

            if version:
                version = "," + version

            role_lines += "{0}{1}\n".format(role, version)

        if self.out_file:
            utils.string_to_file(self.out_file, role_lines)
        else:
            print role_lines

    def get_user_version(self, role):
        """
        Get the user defined version from stdin.
        """
        if self.interactive_version:
            return raw_input("{0}'s version []: ".format(role))

        return ""



import os

import constants as c
import utils as utils


class Config(object):
    """
    Create a config file.
    """
    def __init__(self, args, options, config):
        self.config_path = os.path.join(c.CONFIG_DEFAULT_PATH, c.CONFIG_FILE)
        self.config = config

        # Only load the out file from the options when config was called
        # through the actual CLI, rather than when no default config exists
        if options == {}:
            no_configs_found = True
        else:
            no_configs_found = False

            if options.out_file:
                self.config_path = options.out_file

        if no_configs_found:
            self.show_no_config_help()

        out_config = self.ask_questions()
        self.write_config(out_config)

    def show_no_config_help(self):
        """
        Show a help sentence for new users who have no config.
        """
        print """
    Welcome to ansigenome, you are seeing this message because there
    were no available config files to load.

    Answering the questions below will setup a global config, you
    only need to do this once. You can also exit with CTRL+D.

    This config file will be placed in $HOME/.ansigenome.conf for now.
              """

    def ask_questions(self):
        """
        Obtain config settings from the user.
        """
        out_config = {
            "author": {},
            "license": {},
            "scm": {},
            "options": {},
        }

        for key in c.CONFIG_QUESTIONS:
            section = key[1]
            print
            for question in section:
                answer = utils.ask(question[1], question[2])
                out_config[key[0]][question[0]] = answer

        # Set a few default values.
        test_runner = "https://github.com/nickjj/rolespec"

        out_config["options"]["readme_template"] = ""
        out_config["options"]["travis"] = True
        out_config["options"]["quiet"] = False
        out_config["options"]["dump_with_readme"] = False
        out_config["options"]["test_runner"] = test_runner

        return out_config

    def write_config(self, config):
        """
        Write the config with a little post-converting formatting.
        """
        config_as_string = utils.to_nice_yaml(config)

        config_as_string = "---\n" + config_as_string
        config_as_string = config_as_string.replace("author:", "\nauthor:")
        config_as_string = config_as_string.replace("license:", "\nlicense:")
        config_as_string = config_as_string.replace("scm:", "\nscm:")
        config_as_string = config_as_string.replace("options:", "\noptions:")

        utils.string_to_file(self.config_path, config_as_string)



import os.path

import utils


GIT_AUTHOR = utils.capture_shell("git config user.name")[0][:-1]
GIT_EMAIL = utils.capture_shell("git config user.email")[0][:-1]

MODULE_DIR = os.path.join(os.path.dirname(__file__), os.path.pardir)

VALID_ACTIONS = ("config", "scan", "gendoc", "genmeta",
                 "reqs", "init", "run", "dump")

CONFIG_FILE = ".ansigenome.conf"

CONFIG_DEFAULT_PATH = os.path.expanduser("~")

CONFIG_QUESTIONS = [
    (
        "author", (
            ["name", "What's your name?", GIT_AUTHOR],
            ["company", "Which company do you belong to?", ""],
            ["url", "What's your website?", ""],
            ["email", "What's your e-mail address?", GIT_EMAIL],
            ["twitter", "What's your twitter name without the @?", ""]
        )
    ),
    (
        "license", (
            ["type", "Which license are you using?", "MIT"],
            ["url", "What's the license's URL?",
                    "https://tldrlegal.com/license/mit-license"]
        )
    ),
    (
        "scm", (
            ["type", "Which source control are you using?", "git"],
            ["host", "Which host are you using for SCM?",
                     "https://github.com"],
            ["user", "What's your user name on the above host?", "someone"],
            ["repo_prefix", "How are your roles prefixed in the url?",
                            "ansible-"]
        )

    ),
    (
        "options", (
            ["reqs_format", "Default requirements file format? yml | txt",
                            "yml"],
        ),
    ),
]

CONFIG_MUST_CONTAIN = {
    "author": {
        "name": "str",
        "company": "str",
        "url": "str",
        "email": "str",
        "twitter": "str",
    },
    "license": {
        "type": "str",
        "url": "str",
    },
    "scm": {
        "type": "str",
        "host": "str",
        "user": "str",
        "repo_prefix": "str",
    },
    "options": {
        "readme_template": "str",
        "travis": "bool",
        "quiet": "bool",
        "reqs_format": "str",
        "dump_with_readme": "bool",
        "test_runner": "str",
    },
}

ANSIBLE_FOLDERS = ("defaults", "handlers", "meta",
                   "tasks", "templates", "tests", "vars")

README_TEMPLATE_PATH = os.path.join(MODULE_DIR,
                                    "templates", "README.md.j2")

DEFAULT_META_FILE = """---

dependencies: []

galaxy_info:
  author: '%author_name'
  description: '%role_name described in 1 sentence...'
  company: '%author_company'
  license: '%license_type'
  min_ansible_version: '1.7.1'
  platforms:
    - name: Ubuntu
      versions:
        - precise
        - trusty
    - name: Debian
      versions:
        - wheezy
        - jessie
  categories: [%categories]

ansigenome_info:
  galaxy_id: ''

  travis: True

  synopsis: |
    %role_name described in a few paragraphs....

  usage: |
    Describe how to use %role_name...

  #custom: |
  #  Any custom output you want after the usage section...
"""


LOG_COLOR = {
    "ok": "green",
    "skipped": "cyan",
    "changed": "yellow",
    "missing_readme": "yellow",
    "missing_meta": "red",
}

MESSAGES = {
    "empty_roles_path": "No roles were found at this path:",
    "invalid_config": "'%key' was missing from the following config, please" +
    " fix it:",
    "path_missing": "The following path could not be found:",
    "path_exists": "The following path already exists:",
    "path_unmakable": "The following error occurred when trying to " +
    "create this path:",
    "url_unreachable": "The following url was unreachable:",
    "yaml_error": "%file contains 1 or more syntax errors:",
    "template_error": "%file contains 1 or more syntax errors:",
    "run_success": "%role_count roles were modified with this shell command:",
    "run_error": "There was an error running this shell command:",
    "dump_success": "The role stats were dumped as json to:",
    "help_config": "create a necessary config file to make ansigenome work",
    "help_scan": "scan a path containing Ansible roles and report back " +
    "useful stats",
    "help_gendoc": "generate a README from the meta file for each role",
    "help_genmeta": "augment existing meta files to be compatible with" +
    " Ansigenome",
    "help_reqs": "export a path of roles to a file to be consumed" +
    " by ansible-galaxy install -r",
    "help_init": "init new roles with a custom meta file and tests",
    "help_run": "run shell commands inside of each role's directory",
    "help_dump": "dump a json file containing every stat it gathers from " +
    "the scan path"
}

TEST_PATH = os.path.join(os.path.sep, "tmp", "ansigenome")
